from copy import deepcopy
from sys import version_info
from typing import Any, Dict, Optional, Union
from urllib.parse import quote

from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.serializer import QueryParametersSerializer

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class RequestOptions:
    def __init__(
        self,
        config: BaseConfig,
        headers: Optional[Dict[str, str]] = None,
        query_parameters: Optional[Dict[str, Any]] = None,
        timeouts: Optional[Dict[str, int]] = None,
        data: Optional[Dict[str, Any]] = None,
    ) -> None:
        if headers is None:
            headers = {}
        if query_parameters is None:
            query_parameters = {}
        if timeouts is None:
            timeouts = {}
        self._config = config
        self.headers = headers
        self.query_parameters = {
            quote(k): quote(v)
            for k, v in QueryParametersSerializer(
                query_parameters
            ).query_parameters.items()
        }
        self.timeouts = timeouts
        self.data = data

    def to_dict(self) -> Dict[str, Any]:
        return deepcopy(self.__dict__)

    def to_json(self) -> str:
        return str(self.__dict__)

    def from_dict(self, data: Dict[str, Dict[str, Any]]) -> Self:
        return RequestOptions(
            config=self._config,
            headers=data.get("headers", {}),
            query_parameters=data.get("query_parameters", {}),
            timeouts=data.get("timeouts", {}),
            data=data.get("data", {}),
        )  # pyright: ignore

    def merge(
        self,
        query_parameters: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        data: Optional[str] = None,
        timeouts: Dict[str, int] = {},
        user_request_options: Optional[Union[Self, Dict[str, Any]]] = None,
    ) -> Self:
        """
        Merges the default config values with the user given request options if it exists.
        """

        if query_parameters is None:
            query_parameters = {}
        if headers is None:
            headers = {}
        headers.update(self._config.headers)

        request_options = {
            "headers": headers,
            "query_parameters": query_parameters,
            "timeouts": {
                "read": timeouts.get("read", self._config.read_timeout),
                "write": timeouts.get("write", self._config.write_timeout),
                "connect": timeouts.get("connect", self._config.connect_timeout),
            },
            "data": data,
        }

        if user_request_options is not None:
            if isinstance(user_request_options, RequestOptions):
                _user_request_options = user_request_options.to_dict()
            else:
                _user_request_options = user_request_options

            for key, value in _user_request_options.items():
                if request_options[key] is None:
                    request_options[key] = value
                else:
                    request_options[key].update(value)

        return self.from_dict(request_options)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.log_query import LogQuery

_ALIASES = {
    "timestamp": "timestamp",
    "method": "method",
    "answer_code": "answer_code",
    "query_body": "query_body",
    "answer": "answer",
    "url": "url",
    "ip": "ip",
    "query_headers": "query_headers",
    "sha1": "sha1",
    "nb_api_calls": "nb_api_calls",
    "processing_time_ms": "processing_time_ms",
    "index": "index",
    "var_query_params": "query_params",
    "query_nb_hits": "query_nb_hits",
    "inner_queries": "inner_queries",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Log(BaseModel):
    """
    Log
    """

    timestamp: str
    """ Date and time of the API request, in RFC 3339 format. """
    method: str
    """ HTTP method of the request. """
    answer_code: str
    """ HTTP status code of the response. """
    query_body: str
    """ Request body. """
    answer: str
    """ Response body. """
    url: str
    """ URL of the API endpoint. """
    ip: str
    """ IP address of the client that performed the request. """
    query_headers: str
    """ Request headers (API keys are obfuscated). """
    sha1: str
    """ SHA1 signature of the log entry. """
    nb_api_calls: Optional[str] = None
    """ Number of API requests. """
    processing_time_ms: str
    """ Processing time for the query in milliseconds. This doesn't include latency due to the network.  """
    index: Optional[str] = None
    """ Index targeted by the query. """
    var_query_params: Optional[str] = None
    """ Query parameters sent with the request. """
    query_nb_hits: Optional[str] = None
    """ Number of search results (hits) returned for the query. """
    inner_queries: Optional[List[LogQuery]] = None
    """ Queries performed for the given request. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Log from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Log from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["inner_queries"] = (
            [LogQuery.from_dict(_item) for _item in obj["inner_queries"]]
            if obj.get("inner_queries") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict, field_validator

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.user_highlight_result import UserHighlightResult

_ALIASES = {
    "user_id": "userID",
    "cluster_name": "clusterName",
    "nb_records": "nbRecords",
    "data_size": "dataSize",
    "object_id": "objectID",
    "highlight_result": "_highlightResult",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class UserHit(BaseModel):
    """
    UserHit
    """

    user_id: str
    """ Unique identifier of the user who makes the search request. """
    cluster_name: str
    """ Cluster name. """
    nb_records: int
    """ Number of records in the cluster. """
    data_size: int
    """ Data size taken by all the users assigned to the cluster. """
    object_id: str
    """ userID of the requested user. Same as userID. """
    highlight_result: UserHighlightResult

    @field_validator("user_id")
    def user_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"^[a-zA-Z0-9 \-*.]+$", value):
            raise ValueError(
                r"must validate the regular expression /^[a-zA-Z0-9 \-*.]+$/"
            )
        return value

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UserHit from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UserHit from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["_highlightResult"] = (
            UserHighlightResult.from_dict(obj["_highlightResult"])
            if obj.get("_highlightResult") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.monitoring.models.probes_metric import ProbesMetric

_ALIASES = {
    "cpu_usage": "cpu_usage",
    "ram_indexing_usage": "ram_indexing_usage",
    "ram_search_usage": "ram_search_usage",
    "ssd_usage": "ssd_usage",
    "avg_build_time": "avg_build_time",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Metrics(BaseModel):
    """
    Metrics
    """

    cpu_usage: Optional[Dict[str, List[ProbesMetric]]] = None
    """ CPU idleness in %. """
    ram_indexing_usage: Optional[Dict[str, List[ProbesMetric]]] = None
    """ RAM used for indexing in MB. """
    ram_search_usage: Optional[Dict[str, List[ProbesMetric]]] = None
    """ RAM used for search in MB. """
    ssd_usage: Optional[Dict[str, List[ProbesMetric]]] = None
    """ Solid-state disk (SSD) usage expressed as % of RAM.  0% means no SSD usage. A value of 50% indicates 32&nbsp;GB SSD usage for a machine with 64&nbsp;RAM.  """
    avg_build_time: Optional[Dict[str, List[ProbesMetric]]] = None
    """ Average build time of the indices in seconds. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Metrics from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Metrics from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["cpu_usage"] = dict(
            (
                _k,
                [ProbesMetric.from_dict(_item) for _item in _v]
                if _v is not None
                else None,
            )
            for _k, _v in obj.get("cpu_usage", {}).items()
        )
        obj["ram_indexing_usage"] = dict(
            (
                _k,
                [ProbesMetric.from_dict(_item) for _item in _v]
                if _v is not None
                else None,
            )
            for _k, _v in obj.get("ram_indexing_usage", {}).items()
        )
        obj["ram_search_usage"] = dict(
            (
                _k,
                [ProbesMetric.from_dict(_item) for _item in _v]
                if _v is not None
                else None,
            )
            for _k, _v in obj.get("ram_search_usage", {}).items()
        )
        obj["ssd_usage"] = dict(
            (
                _k,
                [ProbesMetric.from_dict(_item) for _item in _v]
                if _v is not None
                else None,
            )
            for _k, _v in obj.get("ssd_usage", {}).items()
        )
        obj["avg_build_time"] = dict(
            (
                _k,
                [ProbesMetric.from_dict(_item) for _item in _v]
                if _v is not None
                else None,
            )
            for _k, _v in obj.get("avg_build_time", {}).items()
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.ingestion.models.run_outcome import RunOutcome
from algoliasearch.ingestion.models.run_progress import RunProgress
from algoliasearch.ingestion.models.run_reason_code import RunReasonCode
from algoliasearch.ingestion.models.run_status import RunStatus
from algoliasearch.ingestion.models.run_type import RunType

_ALIASES = {
    "run_id": "runID",
    "app_id": "appID",
    "task_id": "taskID",
    "status": "status",
    "progress": "progress",
    "outcome": "outcome",
    "failure_threshold": "failureThreshold",
    "reason": "reason",
    "reason_code": "reasonCode",
    "type": "type",
    "created_at": "createdAt",
    "started_at": "startedAt",
    "finished_at": "finishedAt",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Run(BaseModel):
    """
    Run
    """

    run_id: str
    """ Universally unique identifier (UUID) of a task run. """
    app_id: str
    task_id: str
    """ Universally unique identifier (UUID) of a task. """
    status: RunStatus
    progress: Optional[RunProgress] = None
    outcome: Optional[RunOutcome] = None
    failure_threshold: Optional[int] = None
    """ Maximum accepted percentage of failures for a task run to finish successfully. """
    reason: Optional[str] = None
    """ More information about the task run's outcome. """
    reason_code: Optional[RunReasonCode] = None
    type: RunType
    created_at: str
    """ Date of creation in RFC 3339 format. """
    started_at: Optional[str] = None
    """ Date of start in RFC 3339 format. """
    finished_at: Optional[str] = None
    """ Date of finish in RFC 3339 format. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Run from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Run from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["status"] = obj.get("status")
        obj["progress"] = (
            RunProgress.from_dict(obj["progress"])
            if obj.get("progress") is not None
            else None
        )
        obj["outcome"] = obj.get("outcome")
        obj["reasonCode"] = obj.get("reasonCode")
        obj["type"] = obj.get("type")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.advanced_syntax_features import AdvancedSyntaxFeatures
from algoliasearch.search.models.alternatives_as_exact import AlternativesAsExact
from algoliasearch.search.models.distinct import Distinct
from algoliasearch.search.models.exact_on_single_word_query import (
    ExactOnSingleWordQuery,
)
from algoliasearch.search.models.ignore_plurals import IgnorePlurals
from algoliasearch.search.models.mode import Mode
from algoliasearch.search.models.optional_words import OptionalWords
from algoliasearch.search.models.query_type import QueryType
from algoliasearch.search.models.re_ranking_apply_filter import ReRankingApplyFilter
from algoliasearch.search.models.remove_stop_words import RemoveStopWords
from algoliasearch.search.models.remove_words_if_no_results import (
    RemoveWordsIfNoResults,
)
from algoliasearch.search.models.rendering_content import RenderingContent
from algoliasearch.search.models.semantic_search import SemanticSearch
from algoliasearch.search.models.supported_language import SupportedLanguage
from algoliasearch.search.models.typo_tolerance import TypoTolerance

_ALIASES = {
    "attributes_for_faceting": "attributesForFaceting",
    "replicas": "replicas",
    "pagination_limited_to": "paginationLimitedTo",
    "unretrievable_attributes": "unretrievableAttributes",
    "disable_typo_tolerance_on_words": "disableTypoToleranceOnWords",
    "attributes_to_transliterate": "attributesToTransliterate",
    "camel_case_attributes": "camelCaseAttributes",
    "decompounded_attributes": "decompoundedAttributes",
    "index_languages": "indexLanguages",
    "disable_prefix_on_attributes": "disablePrefixOnAttributes",
    "allow_compression_of_integer_array": "allowCompressionOfIntegerArray",
    "numeric_attributes_for_filtering": "numericAttributesForFiltering",
    "separators_to_index": "separatorsToIndex",
    "searchable_attributes": "searchableAttributes",
    "user_data": "userData",
    "custom_normalization": "customNormalization",
    "attribute_for_distinct": "attributeForDistinct",
    "max_facet_hits": "maxFacetHits",
    "keep_diacritics_on_characters": "keepDiacriticsOnCharacters",
    "custom_ranking": "customRanking",
    "attributes_to_retrieve": "attributesToRetrieve",
    "ranking": "ranking",
    "relevancy_strictness": "relevancyStrictness",
    "attributes_to_highlight": "attributesToHighlight",
    "attributes_to_snippet": "attributesToSnippet",
    "highlight_pre_tag": "highlightPreTag",
    "highlight_post_tag": "highlightPostTag",
    "snippet_ellipsis_text": "snippetEllipsisText",
    "restrict_highlight_and_snippet_arrays": "restrictHighlightAndSnippetArrays",
    "hits_per_page": "hitsPerPage",
    "min_word_sizefor1_typo": "minWordSizefor1Typo",
    "min_word_sizefor2_typos": "minWordSizefor2Typos",
    "typo_tolerance": "typoTolerance",
    "allow_typos_on_numeric_tokens": "allowTyposOnNumericTokens",
    "disable_typo_tolerance_on_attributes": "disableTypoToleranceOnAttributes",
    "ignore_plurals": "ignorePlurals",
    "remove_stop_words": "removeStopWords",
    "query_languages": "queryLanguages",
    "decompound_query": "decompoundQuery",
    "enable_rules": "enableRules",
    "enable_personalization": "enablePersonalization",
    "query_type": "queryType",
    "remove_words_if_no_results": "removeWordsIfNoResults",
    "mode": "mode",
    "semantic_search": "semanticSearch",
    "advanced_syntax": "advancedSyntax",
    "optional_words": "optionalWords",
    "disable_exact_on_attributes": "disableExactOnAttributes",
    "exact_on_single_word_query": "exactOnSingleWordQuery",
    "alternatives_as_exact": "alternativesAsExact",
    "advanced_syntax_features": "advancedSyntaxFeatures",
    "distinct": "distinct",
    "replace_synonyms_in_highlight": "replaceSynonymsInHighlight",
    "min_proximity": "minProximity",
    "response_fields": "responseFields",
    "max_values_per_facet": "maxValuesPerFacet",
    "sort_facet_values_by": "sortFacetValuesBy",
    "attribute_criteria_computed_by_min_proximity": "attributeCriteriaComputedByMinProximity",
    "rendering_content": "renderingContent",
    "enable_re_ranking": "enableReRanking",
    "re_ranking_apply_filter": "reRankingApplyFilter",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class IndexSettings(BaseModel):
    """
    Index settings.
    """

    attributes_for_faceting: Optional[List[str]] = None
    """ Attributes used for [faceting](https://www.algolia.com/doc/guides/managing-results/refine-results/faceting/).  Facets are attributes that let you categorize search results. They can be used for filtering search results. By default, no attribute is used for faceting. Attribute names are case-sensitive.  **Modifiers**  - `filterOnly(\"ATTRIBUTE\")`.   Allows the attribute to be used as a filter but doesn't evaluate the facet values.  - `searchable(\"ATTRIBUTE\")`.   Allows searching for facet values.  - `afterDistinct(\"ATTRIBUTE\")`.   Evaluates the facet count _after_ deduplication with `distinct`.   This ensures accurate facet counts.   You can apply this modifier to searchable facets: `afterDistinct(searchable(ATTRIBUTE))`.  """
    replicas: Optional[List[str]] = None
    """ Creates [replica indices](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/in-depth/replicas/).  Replicas are copies of a primary index with the same records but different settings, synonyms, or rules. If you want to offer a different ranking or sorting of your search results, you'll use replica indices. All index operations on a primary index are automatically forwarded to its replicas. To add a replica index, you must provide the complete set of replicas to this parameter. If you omit a replica from this list, the replica turns into a regular, standalone index that will no longer be synced with the primary index.  **Modifier**  - `virtual(\"REPLICA\")`.   Create a virtual replica,   Virtual replicas don't increase the number of records and are optimized for [Relevant sorting](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/in-depth/relevant-sort/).  """
    pagination_limited_to: Optional[int] = None
    """ Maximum number of search results that can be obtained through pagination.  Higher pagination limits might slow down your search. For pagination limits above 1,000, the sorting of results beyond the 1,000th hit can't be guaranteed.  """
    unretrievable_attributes: Optional[List[str]] = None
    """ Attributes that can't be retrieved at query time.  This can be useful if you want to use an attribute for ranking or to [restrict access](https://www.algolia.com/doc/guides/security/api-keys/how-to/user-restricted-access-to-data/), but don't want to include it in the search results. Attribute names are case-sensitive.  """
    disable_typo_tolerance_on_words: Optional[List[str]] = None
    """ Creates a list of [words which require exact matches](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/in-depth/configuring-typo-tolerance/#turn-off-typo-tolerance-for-certain-words). This also turns off [word splitting and concatenation](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/splitting-and-concatenation/) for the specified words.  """
    attributes_to_transliterate: Optional[List[str]] = None
    """ Attributes, for which you want to support [Japanese transliteration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/#japanese-transliteration-and-type-ahead).  Transliteration supports searching in any of the Japanese writing systems. To support transliteration, you must set the indexing language to Japanese. Attribute names are case-sensitive.  """
    camel_case_attributes: Optional[List[str]] = None
    """ Attributes for which to split [camel case](https://wikipedia.org/wiki/Camel_case) words. Attribute names are case-sensitive.  """
    decompounded_attributes: Optional[object] = None
    """ Searchable attributes to which Algolia should apply [word segmentation](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/how-to/customize-segmentation/) (decompounding). Attribute names are case-sensitive.  Compound words are formed by combining two or more individual words, and are particularly prevalent in Germanic languages—for example, \"firefighter\". With decompounding, the individual components are indexed separately.  You can specify different lists for different languages. Decompounding is supported for these languages: Dutch (`nl`), German (`de`), Finnish (`fi`), Danish (`da`), Swedish (`sv`), and Norwegian (`no`). Decompounding doesn't work for words with [non-spacing mark Unicode characters](https://www.charactercodes.net/category/non-spacing_mark). For example, `Gartenstühle` won't be decompounded if the `ü` consists of `u` (U+0075) and `◌̈` (U+0308).  """
    index_languages: Optional[List[SupportedLanguage]] = None
    """ Languages for language-specific processing steps, such as word detection and dictionary settings.  **You should always specify an indexing language.** If you don't specify an indexing language, the search engine uses all [supported languages](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/supported-languages/), or the languages you specified with the `ignorePlurals` or `removeStopWords` parameters. This can lead to unexpected search results. For more information, see [Language-specific configuration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/).  """
    disable_prefix_on_attributes: Optional[List[str]] = None
    """ Searchable attributes for which you want to turn off [prefix matching](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/override-search-engine-defaults/#adjusting-prefix-search). Attribute names are case-sensitive.  """
    allow_compression_of_integer_array: Optional[bool] = None
    """ Whether arrays with exclusively non-negative integers should be compressed for better performance. If true, the compressed arrays may be reordered.  """
    numeric_attributes_for_filtering: Optional[List[str]] = None
    """ Numeric attributes that can be used as [numerical filters](https://www.algolia.com/doc/guides/managing-results/rules/detecting-intent/how-to/applying-a-custom-filter-for-a-specific-query/#numerical-filters). Attribute names are case-sensitive.  By default, all numeric attributes are available as numerical filters. For faster indexing, reduce the number of numeric attributes.  To turn off filtering for all numeric attributes, specify an attribute that doesn't exist in your index, such as `NO_NUMERIC_FILTERING`.  **Modifier**  - `equalOnly(\"ATTRIBUTE\")`.   Support only filtering based on equality comparisons `=` and `!=`.  """
    separators_to_index: Optional[str] = None
    """ Control which non-alphanumeric characters are indexed.  By default, Algolia ignores [non-alphanumeric characters](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/how-to/how-to-search-in-hyphenated-attributes/#handling-non-alphanumeric-characters) like hyphen (`-`), plus (`+`), and parentheses (`(`,`)`). To include such characters, define them with `separatorsToIndex`.  Separators are all non-letter characters except spaces and currency characters, such as $€£¥.  With `separatorsToIndex`, Algolia treats separator characters as separate words. For example, in a search for \"Disney+\", Algolia considers \"Disney\" and \"+\" as two separate words.  """
    searchable_attributes: Optional[List[str]] = None
    """ Attributes used for searching. Attribute names are case-sensitive.  By default, all attributes are searchable and the [Attribute](https://www.algolia.com/doc/guides/managing-results/relevance-overview/in-depth/ranking-criteria/#attribute) ranking criterion is turned off. With a non-empty list, Algolia only returns results with matches in the selected attributes. In addition, the Attribute ranking criterion is turned on: matches in attributes that are higher in the list of `searchableAttributes` rank first. To make matches in two attributes rank equally, include them in a comma-separated string, such as `\"title,alternate_title\"`. Attributes with the same priority are always unordered.  For more information, see [Searchable attributes](https://www.algolia.com/doc/guides/sending-and-managing-data/prepare-your-data/how-to/setting-searchable-attributes/).  **Modifier**  - `unordered(\"ATTRIBUTE\")`.   Ignore the position of a match within the attribute.  Without a modifier, matches at the beginning of an attribute rank higher than matches at the end.  """
    user_data: Optional[object] = None
    """ An object with custom data.  You can store up to 32kB as custom data.  """
    custom_normalization: Optional[Dict[str, Dict[str, str]]] = None
    """ Characters and their normalized replacements. This overrides Algolia's default [normalization](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/normalization/).  """
    attribute_for_distinct: Optional[str] = None
    """ Attribute that should be used to establish groups of results. Attribute names are case-sensitive.  All records with the same value for this attribute are considered a group. You can combine `attributeForDistinct` with the `distinct` search parameter to control how many items per group are included in the search results.  If you want to use the same attribute also for faceting, use the `afterDistinct` modifier of the `attributesForFaceting` setting. This applies faceting _after_ deduplication, which will result in accurate facet counts.  """
    max_facet_hits: Optional[int] = None
    """ Maximum number of facet values to return when [searching for facet values](https://www.algolia.com/doc/guides/managing-results/refine-results/faceting/#search-for-facet-values). """
    keep_diacritics_on_characters: Optional[str] = None
    """ Characters for which diacritics should be preserved.  By default, Algolia removes diacritics from letters. For example, `é` becomes `e`. If this causes issues in your search, you can specify characters that should keep their diacritics.  """
    custom_ranking: Optional[List[str]] = None
    """ Attributes to use as [custom ranking](https://www.algolia.com/doc/guides/managing-results/must-do/custom-ranking/). Attribute names are case-sensitive.  The custom ranking attributes decide which items are shown first if the other ranking criteria are equal.  Records with missing values for your selected custom ranking attributes are always sorted last. Boolean attributes are sorted based on their alphabetical order.  **Modifiers**  - `asc(\"ATTRIBUTE\")`.   Sort the index by the values of an attribute, in ascending order.  - `desc(\"ATTRIBUTE\")`.   Sort the index by the values of an attribute, in descending order.  If you use two or more custom ranking attributes, [reduce the precision](https://www.algolia.com/doc/guides/managing-results/must-do/custom-ranking/how-to/controlling-custom-ranking-metrics-precision/) of your first attributes, or the other attributes will never be applied.  """
    attributes_to_retrieve: Optional[List[str]] = None
    """ Attributes to include in the API response.  To reduce the size of your response, you can retrieve only some of the attributes. Attribute names are case-sensitive.  - `*` retrieves all attributes, except attributes included in the `customRanking` and `unretrievableAttributes` settings. - To retrieve all attributes except a specific one, prefix the attribute with a dash and combine it with the `*`: `[\"*\", \"-ATTRIBUTE\"]`. - The `objectID` attribute is always included.  """
    ranking: Optional[List[str]] = None
    """ Determines the order in which Algolia returns your results.  By default, each entry corresponds to a [ranking criteria](https://www.algolia.com/doc/guides/managing-results/relevance-overview/in-depth/ranking-criteria/). The tie-breaking algorithm sequentially applies each criterion in the order they're specified. If you configure a replica index for [sorting by an attribute](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/how-to/sort-by-attribute/), you put the sorting attribute at the top of the list.  **Modifiers**  - `asc(\"ATTRIBUTE\")`.   Sort the index by the values of an attribute, in ascending order. - `desc(\"ATTRIBUTE\")`.   Sort the index by the values of an attribute, in descending order.  Before you modify the default setting, you should test your changes in the dashboard, and by [A/B testing](https://www.algolia.com/doc/guides/ab-testing/what-is-ab-testing/).  """
    relevancy_strictness: Optional[int] = None
    """ Relevancy threshold below which less relevant results aren't included in the results.  You can only set `relevancyStrictness` on [virtual replica indices](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/in-depth/replicas/#what-are-virtual-replicas). Use this setting to strike a balance between the relevance and number of returned results.  """
    attributes_to_highlight: Optional[List[str]] = None
    """ Attributes to highlight.  By default, all searchable attributes are highlighted. Use `*` to highlight all attributes or use an empty array `[]` to turn off highlighting. Attribute names are case-sensitive.  With highlighting, strings that match the search query are surrounded by HTML tags defined by `highlightPreTag` and `highlightPostTag`. You can use this to visually highlight matching parts of a search query in your UI.  For more information, see [Highlighting and snippeting](https://www.algolia.com/doc/guides/building-search-ui/ui-and-ux-patterns/highlighting-snippeting/js/).  """
    attributes_to_snippet: Optional[List[str]] = None
    """ Attributes for which to enable snippets. Attribute names are case-sensitive.  Snippets provide additional context to matched words. If you enable snippets, they include 10 words, including the matched word. The matched word will also be wrapped by HTML tags for highlighting. You can adjust the number of words with the following notation: `ATTRIBUTE:NUMBER`, where `NUMBER` is the number of words to be extracted.  """
    highlight_pre_tag: Optional[str] = None
    """ HTML tag to insert before the highlighted parts in all highlighted results and snippets. """
    highlight_post_tag: Optional[str] = None
    """ HTML tag to insert after the highlighted parts in all highlighted results and snippets. """
    snippet_ellipsis_text: Optional[str] = None
    """ String used as an ellipsis indicator when a snippet is truncated. """
    restrict_highlight_and_snippet_arrays: Optional[bool] = None
    """ Whether to restrict highlighting and snippeting to items that at least partially matched the search query. By default, all items are highlighted and snippeted.  """
    hits_per_page: Optional[int] = None
    """ Number of hits per page. """
    min_word_sizefor1_typo: Optional[int] = None
    """ Minimum number of characters a word in the search query must contain to accept matches with [one typo](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/in-depth/configuring-typo-tolerance/#configuring-word-length-for-typos). """
    min_word_sizefor2_typos: Optional[int] = None
    """ Minimum number of characters a word in the search query must contain to accept matches with [two typos](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/in-depth/configuring-typo-tolerance/#configuring-word-length-for-typos). """
    typo_tolerance: Optional[TypoTolerance] = None
    allow_typos_on_numeric_tokens: Optional[bool] = None
    """ Whether to allow typos on numbers in the search query.  Turn off this setting to reduce the number of irrelevant matches when searching in large sets of similar numbers.  """
    disable_typo_tolerance_on_attributes: Optional[List[str]] = None
    """ Attributes for which you want to turn off [typo tolerance](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/). Attribute names are case-sensitive.  Returning only exact matches can help when:  - [Searching in hyphenated attributes](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/how-to/how-to-search-in-hyphenated-attributes/). - Reducing the number of matches when you have too many.   This can happen with attributes that are long blocks of text, such as product descriptions.  Consider alternatives such as `disableTypoToleranceOnWords` or adding synonyms if your attributes have intentional unusual spellings that might look like typos.  """
    ignore_plurals: Optional[IgnorePlurals] = None
    remove_stop_words: Optional[RemoveStopWords] = None
    query_languages: Optional[List[SupportedLanguage]] = None
    """ Languages for language-specific query processing steps such as plurals, stop-word removal, and word-detection dictionaries.  This setting sets a default list of languages used by the `removeStopWords` and `ignorePlurals` settings. This setting also sets a dictionary for word detection in the logogram-based [CJK](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/normalization/#normalization-for-logogram-based-languages-cjk) languages. To support this, you must place the CJK language **first**.  **You should always specify a query language.** If you don't specify an indexing language, the search engine uses all [supported languages](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/supported-languages/), or the languages you specified with the `ignorePlurals` or `removeStopWords` parameters. This can lead to unexpected search results. For more information, see [Language-specific configuration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/).  """
    decompound_query: Optional[bool] = None
    """ Whether to split compound words in the query into their building blocks.  For more information, see [Word segmentation](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/#splitting-compound-words). Word segmentation is supported for these languages: German, Dutch, Finnish, Swedish, and Norwegian. Decompounding doesn't work for words with [non-spacing mark Unicode characters](https://www.charactercodes.net/category/non-spacing_mark). For example, `Gartenstühle` won't be decompounded if the `ü` consists of `u` (U+0075) and `◌̈` (U+0308).  """
    enable_rules: Optional[bool] = None
    """ Whether to enable rules. """
    enable_personalization: Optional[bool] = None
    """ Whether to enable Personalization. """
    query_type: Optional[QueryType] = None
    remove_words_if_no_results: Optional[RemoveWordsIfNoResults] = None
    mode: Optional[Mode] = None
    semantic_search: Optional[SemanticSearch] = None
    advanced_syntax: Optional[bool] = None
    """ Whether to support phrase matching and excluding words from search queries.  Use the `advancedSyntaxFeatures` parameter to control which feature is supported.  """
    optional_words: Optional[OptionalWords] = None
    disable_exact_on_attributes: Optional[List[str]] = None
    """ Searchable attributes for which you want to [turn off the Exact ranking criterion](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/override-search-engine-defaults/in-depth/adjust-exact-settings/#turn-off-exact-for-some-attributes). Attribute names are case-sensitive.  This can be useful for attributes with long values, where the likelihood of an exact match is high, such as product descriptions. Turning off the Exact ranking criterion for these attributes favors exact matching on other attributes. This reduces the impact of individual attributes with a lot of content on ranking.  """
    exact_on_single_word_query: Optional[ExactOnSingleWordQuery] = None
    alternatives_as_exact: Optional[List[AlternativesAsExact]] = None
    """ Determine which plurals and synonyms should be considered an exact matches.  By default, Algolia treats singular and plural forms of a word, and single-word synonyms, as [exact](https://www.algolia.com/doc/guides/managing-results/relevance-overview/in-depth/ranking-criteria/#exact) matches when searching. For example:  - \"swimsuit\" and \"swimsuits\" are treated the same - \"swimsuit\" and \"swimwear\" are treated the same (if they are [synonyms](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/#regular-synonyms)).  - `ignorePlurals`.   Plurals and similar declensions added by the `ignorePlurals` setting are considered exact matches.  - `singleWordSynonym`.   Single-word synonyms, such as \"NY\" = \"NYC\", are considered exact matches.  - `multiWordsSynonym`.   Multi-word synonyms, such as \"NY\" = \"New York\", are considered exact matches.  """
    advanced_syntax_features: Optional[List[AdvancedSyntaxFeatures]] = None
    """ Advanced search syntax features you want to support.  - `exactPhrase`.   Phrases in quotes must match exactly.   For example, `sparkly blue \"iPhone case\"` only returns records with the exact string \"iPhone case\".  - `excludeWords`.   Query words prefixed with a `-` must not occur in a record.   For example, `search -engine` matches records that contain \"search\" but not \"engine\".  This setting only has an effect if `advancedSyntax` is true.  """
    distinct: Optional[Distinct] = None
    replace_synonyms_in_highlight: Optional[bool] = None
    """ Whether to replace a highlighted word with the matched synonym.  By default, the original words are highlighted even if a synonym matches. For example, with `home` as a synonym for `house` and a search for `home`, records matching either \"home\" or \"house\" are included in the search results, and either \"home\" or \"house\" are highlighted.  With `replaceSynonymsInHighlight` set to `true`, a search for `home` still matches the same records, but all occurrences of \"house\" are replaced by \"home\" in the highlighted response.  """
    min_proximity: Optional[int] = None
    """ Minimum proximity score for two matching words.  This adjusts the [Proximity ranking criterion](https://www.algolia.com/doc/guides/managing-results/relevance-overview/in-depth/ranking-criteria/#proximity) by equally scoring matches that are farther apart.  For example, if `minProximity` is 2, neighboring matches and matches with one word between them would have the same score.  """
    response_fields: Optional[List[str]] = None
    """ Properties to include in the API response of search and browse requests.  By default, all response properties are included. To reduce the response size, you can select which properties should be included.  An empty list may lead to an empty API response (except properties you can't exclude).  You can't exclude these properties: `message`, `warning`, `cursor`, `abTestVariantID`, or any property added by setting `getRankingInfo` to true.  Your search depends on the `hits` field. If you omit this field, searches won't return any results. Your UI might also depend on other properties, for example, for pagination. Before restricting the response size, check the impact on your search experience.  """
    max_values_per_facet: Optional[int] = None
    """ Maximum number of facet values to return for each facet. """
    sort_facet_values_by: Optional[str] = None
    """ Order in which to retrieve facet values.  - `count`.   Facet values are retrieved by decreasing count.   The count is the number of matching records containing this facet value.  - `alpha`.   Retrieve facet values alphabetically.  This setting doesn't influence how facet values are displayed in your UI (see `renderingContent`). For more information, see [facet value display](https://www.algolia.com/doc/guides/building-search-ui/ui-and-ux-patterns/facet-display/js/).  """
    attribute_criteria_computed_by_min_proximity: Optional[bool] = None
    """ Whether the best matching attribute should be determined by minimum proximity.  This setting only affects ranking if the Attribute ranking criterion comes before Proximity in the `ranking` setting. If true, the best matching attribute is selected based on the minimum proximity of multiple matches. Otherwise, the best matching attribute is determined by the order in the `searchableAttributes` setting.  """
    rendering_content: Optional[RenderingContent] = None
    enable_re_ranking: Optional[bool] = None
    """ Whether this search will use [Dynamic Re-Ranking](https://www.algolia.com/doc/guides/algolia-ai/re-ranking/).  This setting only has an effect if you activated Dynamic Re-Ranking for this index in the Algolia dashboard.  """
    re_ranking_apply_filter: Optional[ReRankingApplyFilter] = None

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of IndexSettings from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of IndexSettings from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["indexLanguages"] = obj.get("indexLanguages")
        obj["typoTolerance"] = (
            TypoTolerance.from_dict(obj["typoTolerance"])
            if obj.get("typoTolerance") is not None
            else None
        )
        obj["ignorePlurals"] = (
            IgnorePlurals.from_dict(obj["ignorePlurals"])
            if obj.get("ignorePlurals") is not None
            else None
        )
        obj["removeStopWords"] = (
            RemoveStopWords.from_dict(obj["removeStopWords"])
            if obj.get("removeStopWords") is not None
            else None
        )
        obj["queryLanguages"] = obj.get("queryLanguages")
        obj["queryType"] = obj.get("queryType")
        obj["removeWordsIfNoResults"] = obj.get("removeWordsIfNoResults")
        obj["mode"] = obj.get("mode")
        obj["semanticSearch"] = (
            SemanticSearch.from_dict(obj["semanticSearch"])
            if obj.get("semanticSearch") is not None
            else None
        )
        obj["optionalWords"] = (
            OptionalWords.from_dict(obj["optionalWords"])
            if obj.get("optionalWords") is not None
            else None
        )
        obj["exactOnSingleWordQuery"] = obj.get("exactOnSingleWordQuery")
        obj["alternativesAsExact"] = obj.get("alternativesAsExact")
        obj["advancedSyntaxFeatures"] = obj.get("advancedSyntaxFeatures")
        obj["distinct"] = (
            Distinct.from_dict(obj["distinct"])
            if obj.get("distinct") is not None
            else None
        )
        obj["renderingContent"] = (
            RenderingContent.from_dict(obj["renderingContent"])
            if obj.get("renderingContent") is not None
            else None
        )
        obj["reRankingApplyFilter"] = (
            ReRankingApplyFilter.from_dict(obj["reRankingApplyFilter"])
            if obj.get("reRankingApplyFilter") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .click_position import ClickPosition
from .currency_code import CurrencyCode
from .daily_add_to_cart_rates import DailyAddToCartRates
from .daily_average_clicks import DailyAverageClicks
from .daily_click_through_rates import DailyClickThroughRates
from .daily_conversion_rates import DailyConversionRates
from .daily_no_click_rates import DailyNoClickRates
from .daily_no_results_rates import DailyNoResultsRates
from .daily_purchase_rates import DailyPurchaseRates
from .daily_revenue import DailyRevenue
from .daily_searches import DailySearches
from .daily_searches_no_clicks import DailySearchesNoClicks
from .daily_searches_no_results import DailySearchesNoResults
from .daily_users import DailyUsers
from .direction import Direction
from .error_base import ErrorBase
from .get_add_to_cart_rate_response import GetAddToCartRateResponse
from .get_average_click_position_response import GetAverageClickPositionResponse
from .get_click_positions_response import GetClickPositionsResponse
from .get_click_through_rate_response import GetClickThroughRateResponse
from .get_conversion_rate_response import GetConversionRateResponse
from .get_no_click_rate_response import GetNoClickRateResponse
from .get_no_results_rate_response import GetNoResultsRateResponse
from .get_purchase_rate_response import GetPurchaseRateResponse
from .get_revenue import GetRevenue
from .get_searches_count_response import GetSearchesCountResponse
from .get_searches_no_clicks_response import GetSearchesNoClicksResponse
from .get_searches_no_results_response import GetSearchesNoResultsResponse
from .get_status_response import GetStatusResponse
from .get_top_countries_response import GetTopCountriesResponse
from .get_top_filter_attribute import GetTopFilterAttribute
from .get_top_filter_attributes_response import GetTopFilterAttributesResponse
from .get_top_filter_for_attribute import GetTopFilterForAttribute
from .get_top_filter_for_attribute_response import GetTopFilterForAttributeResponse
from .get_top_filters_no_results_response import GetTopFiltersNoResultsResponse
from .get_top_filters_no_results_value import GetTopFiltersNoResultsValue
from .get_top_filters_no_results_values import GetTopFiltersNoResultsValues
from .get_top_hits_response import GetTopHitsResponse
from .get_top_searches_response import GetTopSearchesResponse
from .get_users_count_response import GetUsersCountResponse
from .operator import Operator
from .order_by import OrderBy
from .top_country import TopCountry
from .top_hit import TopHit
from .top_hit_with_analytics import TopHitWithAnalytics
from .top_hit_with_revenue_analytics import TopHitWithRevenueAnalytics
from .top_hits_response import TopHitsResponse
from .top_hits_response_with_analytics import TopHitsResponseWithAnalytics
from .top_hits_response_with_revenue_analytics import (
    TopHitsResponseWithRevenueAnalytics,
)
from .top_search import TopSearch
from .top_search_with_analytics import TopSearchWithAnalytics
from .top_search_with_revenue_analytics import TopSearchWithRevenueAnalytics
from .top_searches_response import TopSearchesResponse
from .top_searches_response_with_analytics import TopSearchesResponseWithAnalytics
from .top_searches_response_with_revenue_analytics import (
    TopSearchesResponseWithRevenueAnalytics,
)

__all__ = (
    "ClickPosition",
    "CurrencyCode",
    "DailyAddToCartRates",
    "DailyAverageClicks",
    "DailyClickThroughRates",
    "DailyConversionRates",
    "DailyNoClickRates",
    "DailyNoResultsRates",
    "DailyPurchaseRates",
    "DailyRevenue",
    "DailySearches",
    "DailySearchesNoClicks",
    "DailySearchesNoResults",
    "DailyUsers",
    "Direction",
    "ErrorBase",
    "GetAddToCartRateResponse",
    "GetAverageClickPositionResponse",
    "GetClickPositionsResponse",
    "GetClickThroughRateResponse",
    "GetConversionRateResponse",
    "GetNoClickRateResponse",
    "GetNoResultsRateResponse",
    "GetPurchaseRateResponse",
    "GetRevenue",
    "GetSearchesCountResponse",
    "GetSearchesNoClicksResponse",
    "GetSearchesNoResultsResponse",
    "GetStatusResponse",
    "GetTopCountriesResponse",
    "GetTopFilterAttribute",
    "GetTopFilterAttributesResponse",
    "GetTopFilterForAttribute",
    "GetTopFilterForAttributeResponse",
    "GetTopFiltersNoResultsResponse",
    "GetTopFiltersNoResultsValue",
    "GetTopFiltersNoResultsValues",
    "GetTopHitsResponse",
    "GetTopSearchesResponse",
    "GetUsersCountResponse",
    "Operator",
    "OrderBy",
    "TopCountry",
    "TopHit",
    "TopHitWithAnalytics",
    "TopHitWithRevenueAnalytics",
    "TopHitsResponse",
    "TopHitsResponseWithAnalytics",
    "TopHitsResponseWithRevenueAnalytics",
    "TopSearch",
    "TopSearchWithAnalytics",
    "TopSearchWithRevenueAnalytics",
    "TopSearchesResponse",
    "TopSearchesResponseWithAnalytics",
    "TopSearchesResponseWithRevenueAnalytics",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps
from sys import version_info
from typing import Any, Dict, Optional, Union
from urllib.parse import quote

from pydantic import Field, StrictInt, StrictStr
from typing_extensions import Annotated

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self

from algoliasearch.abtesting.config import AbtestingConfig
from algoliasearch.abtesting.models import (
    ABTest,
    ABTestResponse,
    AddABTestsRequest,
    EstimateABTestRequest,
    EstimateABTestResponse,
    ListABTestsResponse,
    ScheduleABTestResponse,
    ScheduleABTestsRequest,
)
from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.serializer import body_serializer
from algoliasearch.http.transporter import Transporter
from algoliasearch.http.transporter_sync import TransporterSync
from algoliasearch.http.verb import Verb


class AbtestingClient:
    """The Algolia 'AbtestingClient' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.
    region ("de" | "us"): The region of your Algolia application.

    Returns:
    The initialized API client.

    Example:
    _client = AbtestingClient("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY", region="'de' or 'us'")
    _client_with_named_args = AbtestingClient(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY", region="'de' or 'us'")

    See `AbtestingClient.create_with_config` for advanced configuration.
    """

    _transporter: Transporter
    _config: BaseConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        region: Optional[str] = None,
        transporter: Optional[Transporter] = None,
        config: Optional[AbtestingConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = AbtestingConfig(
                transporter.config.app_id, transporter.config.api_key, region
            )
        elif config is None:
            config = AbtestingConfig(app_id, api_key, region)

        config.set_default_hosts()

        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = Transporter(config)
        self._transporter = transporter

    @classmethod
    def create_with_config(
        cls, config: AbtestingConfig, transporter: Optional[Transporter] = None
    ) -> AbtestingClient:
        """Allows creating a client with a customized `AbtestingConfig` and `Transporter`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (AbtestingConfig): The config of the API client.
        transporter (Transporter): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = AbtestingClient.create_with_config(config=AbtestingConfig(...))
        _client_with_custom_config_and_transporter = AbtestingClient.create_with_config(config=AbtestingConfig(...), transporter=Transporter(...))
        """
        if transporter is None:
            transporter = Transporter(config)

        client = AbtestingClient(
            app_id=config.app_id,
            api_key=config.api_key,
            region=config.region,
            transporter=transporter,
            config=config,
        )

        return client

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        await self.close()

    async def close(self) -> None:
        """Closes the underlying `transporter` of the API client."""
        return await self._transporter.close()

    async def set_client_api_key(self, api_key: str) -> None:
        """Sets a new API key to authenticate requests."""
        self._transporter.config.set_client_api_key(api_key)

    async def add_user_agent(self, segment: str, version: Optional[str] = None) -> None:
        """adds a segment to the default user agent, and update the headers sent with each requests as well"""
        self._transporter.config.add_user_agent(segment, version)

    async def add_ab_tests_with_http_info(
        self,
        add_ab_tests_request: Union[AddABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Creates a new A/B test.

        Required API Key ACLs:
          - editSettings

        :param add_ab_tests_request: (required)
        :type add_ab_tests_request: AddABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if add_ab_tests_request is None:
            raise ValueError(
                "Parameter `add_ab_tests_request` is required when calling `add_ab_tests`."
            )

        _data = {}
        if add_ab_tests_request is not None:
            _data = add_ab_tests_request

        return await self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def add_ab_tests(
        self,
        add_ab_tests_request: Union[AddABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Creates a new A/B test.

        Required API Key ACLs:
          - editSettings

        :param add_ab_tests_request: (required)
        :type add_ab_tests_request: AddABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = await self.add_ab_tests_with_http_info(
            add_ab_tests_request, request_options
        )
        return resp.deserialize(ABTestResponse, resp.raw_data)

    async def custom_delete_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError(
                "Parameter `path` is required when calling `custom_delete`."
            )

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_delete(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = await self.custom_delete_with_http_info(
            path, parameters, request_options
        )
        return resp.deserialize(object, resp.raw_data)

    async def custom_get_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_get`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        return await self._transporter.request(
            verb=Verb.GET,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_get(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = await self.custom_get_with_http_info(path, parameters, request_options)
        return resp.deserialize(object, resp.raw_data)

    async def custom_post_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_post`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.POST,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_post(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = await self.custom_post_with_http_info(
            path, parameters, body, request_options
        )
        return resp.deserialize(object, resp.raw_data)

    async def custom_put_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_put`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.PUT,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_put(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = await self.custom_put_with_http_info(
            path, parameters, body, request_options
        )
        return resp.deserialize(object, resp.raw_data)

    async def delete_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Deletes an A/B test by its ID.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError(
                "Parameter `id` is required when calling `delete_ab_test`."
            )

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/2/abtests/{id}".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def delete_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Deletes an A/B test by its ID.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = await self.delete_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTestResponse, resp.raw_data)

    async def estimate_ab_test_with_http_info(
        self,
        estimate_ab_test_request: Union[EstimateABTestRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Given the traffic percentage and the expected effect size, this endpoint estimates the sample size and duration of an A/B test based on historical traffic.

        Required API Key ACLs:
          - analytics

        :param estimate_ab_test_request: (required)
        :type estimate_ab_test_request: EstimateABTestRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if estimate_ab_test_request is None:
            raise ValueError(
                "Parameter `estimate_ab_test_request` is required when calling `estimate_ab_test`."
            )

        _data = {}
        if estimate_ab_test_request is not None:
            _data = estimate_ab_test_request

        return await self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/estimate",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def estimate_ab_test(
        self,
        estimate_ab_test_request: Union[EstimateABTestRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> EstimateABTestResponse:
        """
        Given the traffic percentage and the expected effect size, this endpoint estimates the sample size and duration of an A/B test based on historical traffic.

        Required API Key ACLs:
          - analytics

        :param estimate_ab_test_request: (required)
        :type estimate_ab_test_request: EstimateABTestRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'EstimateABTestResponse' result object.
        """
        resp = await self.estimate_ab_test_with_http_info(
            estimate_ab_test_request, request_options
        )
        return resp.deserialize(EstimateABTestResponse, resp.raw_data)

    async def get_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Retrieves the details for an A/B test by its ID.

        Required API Key ACLs:
          - analytics

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError("Parameter `id` is required when calling `get_ab_test`.")

        return await self._transporter.request(
            verb=Verb.GET,
            path="/2/abtests/{id}".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def get_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTest:
        """
        Retrieves the details for an A/B test by its ID.

        Required API Key ACLs:
          - analytics

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTest' result object.
        """
        resp = await self.get_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTest, resp.raw_data)

    async def list_ab_tests_with_http_info(
        self,
        offset: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=0)]],
            Field(description="Position of the first item to return."),
        ] = None,
        limit: Annotated[
            Optional[StrictInt], Field(description="Number of items to return.")
        ] = None,
        index_prefix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name prefix. Only A/B tests for indices starting with this string are included in the response."
            ),
        ] = None,
        index_suffix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name suffix. Only A/B tests for indices ending with this string are included in the response."
            ),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Lists all A/B tests you configured for this application.

        Required API Key ACLs:
          - analytics

        :param offset: Position of the first item to return.
        :type offset: int
        :param limit: Number of items to return.
        :type limit: int
        :param index_prefix: Index name prefix. Only A/B tests for indices starting with this string are included in the response.
        :type index_prefix: str
        :param index_suffix: Index name suffix. Only A/B tests for indices ending with this string are included in the response.
        :type index_suffix: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        _query_parameters: Dict[str, Any] = {}

        if offset is not None:
            _query_parameters["offset"] = offset
        if limit is not None:
            _query_parameters["limit"] = limit
        if index_prefix is not None:
            _query_parameters["indexPrefix"] = index_prefix
        if index_suffix is not None:
            _query_parameters["indexSuffix"] = index_suffix

        return await self._transporter.request(
            verb=Verb.GET,
            path="/2/abtests",
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def list_ab_tests(
        self,
        offset: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=0)]],
            Field(description="Position of the first item to return."),
        ] = None,
        limit: Annotated[
            Optional[StrictInt], Field(description="Number of items to return.")
        ] = None,
        index_prefix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name prefix. Only A/B tests for indices starting with this string are included in the response."
            ),
        ] = None,
        index_suffix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name suffix. Only A/B tests for indices ending with this string are included in the response."
            ),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ListABTestsResponse:
        """
        Lists all A/B tests you configured for this application.

        Required API Key ACLs:
          - analytics

        :param offset: Position of the first item to return.
        :type offset: int
        :param limit: Number of items to return.
        :type limit: int
        :param index_prefix: Index name prefix. Only A/B tests for indices starting with this string are included in the response.
        :type index_prefix: str
        :param index_suffix: Index name suffix. Only A/B tests for indices ending with this string are included in the response.
        :type index_suffix: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ListABTestsResponse' result object.
        """
        resp = await self.list_ab_tests_with_http_info(
            offset, limit, index_prefix, index_suffix, request_options
        )
        return resp.deserialize(ListABTestsResponse, resp.raw_data)

    async def schedule_ab_test_with_http_info(
        self,
        schedule_ab_tests_request: Union[ScheduleABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Schedule an A/B test to be started at a later time.

        Required API Key ACLs:
          - editSettings

        :param schedule_ab_tests_request: (required)
        :type schedule_ab_tests_request: ScheduleABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if schedule_ab_tests_request is None:
            raise ValueError(
                "Parameter `schedule_ab_tests_request` is required when calling `schedule_ab_test`."
            )

        _data = {}
        if schedule_ab_tests_request is not None:
            _data = schedule_ab_tests_request

        return await self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/schedule",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def schedule_ab_test(
        self,
        schedule_ab_tests_request: Union[ScheduleABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ScheduleABTestResponse:
        """
        Schedule an A/B test to be started at a later time.

        Required API Key ACLs:
          - editSettings

        :param schedule_ab_tests_request: (required)
        :type schedule_ab_tests_request: ScheduleABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ScheduleABTestResponse' result object.
        """
        resp = await self.schedule_ab_test_with_http_info(
            schedule_ab_tests_request, request_options
        )
        return resp.deserialize(ScheduleABTestResponse, resp.raw_data)

    async def stop_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Stops an A/B test by its ID.  You can't restart stopped A/B tests.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError("Parameter `id` is required when calling `stop_ab_test`.")

        return await self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/{id}/stop".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def stop_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Stops an A/B test by its ID.  You can't restart stopped A/B tests.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = await self.stop_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTestResponse, resp.raw_data)


class AbtestingClientSync:
    """The Algolia 'AbtestingClientSync' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.
    region ("de" | "us"): The region of your Algolia application.

    Returns:
    The initialized API client.

    Example:
    _client = AbtestingClientSync("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY", region="'de' or 'us'")
    _client_with_named_args = AbtestingClientSync(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY", region="'de' or 'us'")

    See `AbtestingClientSync.create_with_config` for advanced configuration.
    """

    _transporter: TransporterSync
    _config: BaseConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        region: Optional[str] = None,
        transporter: Optional[TransporterSync] = None,
        config: Optional[AbtestingConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = AbtestingConfig(
                transporter.config.app_id, transporter.config.api_key, region
            )
        elif config is None:
            config = AbtestingConfig(app_id, api_key, region)

        config.set_default_hosts()

        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = TransporterSync(config)
        self._transporter = transporter

    @classmethod
    def create_with_config(
        cls, config: AbtestingConfig, transporter: Optional[TransporterSync] = None
    ) -> AbtestingClientSync:
        """Allows creating a client with a customized `AbtestingConfig` and `TransporterSync`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (AbtestingConfig): The config of the API client.
        transporter (TransporterSync): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = AbtestingClientSync.create_with_config(config=AbtestingConfig(...))
        _client_with_custom_config_and_transporter = AbtestingClientSync.create_with_config(config=AbtestingConfig(...), transporter=TransporterSync(...))
        """
        if transporter is None:
            transporter = TransporterSync(config)

        client = AbtestingClientSync(
            app_id=config.app_id,
            api_key=config.api_key,
            region=config.region,
            transporter=transporter,
            config=config,
        )

        return client

    def __enter__(self) -> Self:
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        self.close()

    def close(self) -> None:
        return self._transporter.close()

    def set_client_api_key(self, api_key: str) -> None:
        """Sets a new API key to authenticate requests."""
        self._transporter.config.set_client_api_key(api_key)

    def add_user_agent(self, segment: str, version: Optional[str] = None) -> None:
        """adds a segment to the default user agent, and update the headers sent with each requests as well"""
        self._transporter.config.add_user_agent(segment, version)

    def add_ab_tests_with_http_info(
        self,
        add_ab_tests_request: Union[AddABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Creates a new A/B test.

        Required API Key ACLs:
          - editSettings

        :param add_ab_tests_request: (required)
        :type add_ab_tests_request: AddABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if add_ab_tests_request is None:
            raise ValueError(
                "Parameter `add_ab_tests_request` is required when calling `add_ab_tests`."
            )

        _data = {}
        if add_ab_tests_request is not None:
            _data = add_ab_tests_request

        return self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def add_ab_tests(
        self,
        add_ab_tests_request: Union[AddABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Creates a new A/B test.

        Required API Key ACLs:
          - editSettings

        :param add_ab_tests_request: (required)
        :type add_ab_tests_request: AddABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = self.add_ab_tests_with_http_info(add_ab_tests_request, request_options)
        return resp.deserialize(ABTestResponse, resp.raw_data)

    def custom_delete_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError(
                "Parameter `path` is required when calling `custom_delete`."
            )

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        return self._transporter.request(
            verb=Verb.DELETE,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def custom_delete(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = self.custom_delete_with_http_info(path, parameters, request_options)
        return resp.deserialize(object, resp.raw_data)

    def custom_get_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_get`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        return self._transporter.request(
            verb=Verb.GET,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def custom_get(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = self.custom_get_with_http_info(path, parameters, request_options)
        return resp.deserialize(object, resp.raw_data)

    def custom_post_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_post`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        _data = {}
        if body is not None:
            _data = body

        return self._transporter.request(
            verb=Verb.POST,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def custom_post(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = self.custom_post_with_http_info(path, parameters, body, request_options)
        return resp.deserialize(object, resp.raw_data)

    def custom_put_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_put`.")

        _query_parameters: Dict[str, Any] = {}

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters[_qpkey] = _qpvalue

        _data = {}
        if body is not None:
            _data = body

        return self._transporter.request(
            verb=Verb.PUT,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def custom_put(
        self,
        path: Annotated[
            StrictStr,
            Field(description="Path of the endpoint, for example `1/newFeature`."),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method lets you send requests to the Algolia REST API.


        :param path: Path of the endpoint, for example `1/newFeature`. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        resp = self.custom_put_with_http_info(path, parameters, body, request_options)
        return resp.deserialize(object, resp.raw_data)

    def delete_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Deletes an A/B test by its ID.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError(
                "Parameter `id` is required when calling `delete_ab_test`."
            )

        return self._transporter.request(
            verb=Verb.DELETE,
            path="/2/abtests/{id}".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def delete_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Deletes an A/B test by its ID.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = self.delete_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTestResponse, resp.raw_data)

    def estimate_ab_test_with_http_info(
        self,
        estimate_ab_test_request: Union[EstimateABTestRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Given the traffic percentage and the expected effect size, this endpoint estimates the sample size and duration of an A/B test based on historical traffic.

        Required API Key ACLs:
          - analytics

        :param estimate_ab_test_request: (required)
        :type estimate_ab_test_request: EstimateABTestRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if estimate_ab_test_request is None:
            raise ValueError(
                "Parameter `estimate_ab_test_request` is required when calling `estimate_ab_test`."
            )

        _data = {}
        if estimate_ab_test_request is not None:
            _data = estimate_ab_test_request

        return self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/estimate",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def estimate_ab_test(
        self,
        estimate_ab_test_request: Union[EstimateABTestRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> EstimateABTestResponse:
        """
        Given the traffic percentage and the expected effect size, this endpoint estimates the sample size and duration of an A/B test based on historical traffic.

        Required API Key ACLs:
          - analytics

        :param estimate_ab_test_request: (required)
        :type estimate_ab_test_request: EstimateABTestRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'EstimateABTestResponse' result object.
        """
        resp = self.estimate_ab_test_with_http_info(
            estimate_ab_test_request, request_options
        )
        return resp.deserialize(EstimateABTestResponse, resp.raw_data)

    def get_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Retrieves the details for an A/B test by its ID.

        Required API Key ACLs:
          - analytics

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError("Parameter `id` is required when calling `get_ab_test`.")

        return self._transporter.request(
            verb=Verb.GET,
            path="/2/abtests/{id}".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def get_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTest:
        """
        Retrieves the details for an A/B test by its ID.

        Required API Key ACLs:
          - analytics

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTest' result object.
        """
        resp = self.get_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTest, resp.raw_data)

    def list_ab_tests_with_http_info(
        self,
        offset: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=0)]],
            Field(description="Position of the first item to return."),
        ] = None,
        limit: Annotated[
            Optional[StrictInt], Field(description="Number of items to return.")
        ] = None,
        index_prefix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name prefix. Only A/B tests for indices starting with this string are included in the response."
            ),
        ] = None,
        index_suffix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name suffix. Only A/B tests for indices ending with this string are included in the response."
            ),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Lists all A/B tests you configured for this application.

        Required API Key ACLs:
          - analytics

        :param offset: Position of the first item to return.
        :type offset: int
        :param limit: Number of items to return.
        :type limit: int
        :param index_prefix: Index name prefix. Only A/B tests for indices starting with this string are included in the response.
        :type index_prefix: str
        :param index_suffix: Index name suffix. Only A/B tests for indices ending with this string are included in the response.
        :type index_suffix: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        _query_parameters: Dict[str, Any] = {}

        if offset is not None:
            _query_parameters["offset"] = offset
        if limit is not None:
            _query_parameters["limit"] = limit
        if index_prefix is not None:
            _query_parameters["indexPrefix"] = index_prefix
        if index_suffix is not None:
            _query_parameters["indexSuffix"] = index_suffix

        return self._transporter.request(
            verb=Verb.GET,
            path="/2/abtests",
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def list_ab_tests(
        self,
        offset: Annotated[
            Optional[Annotated[int, Field(strict=True, ge=0)]],
            Field(description="Position of the first item to return."),
        ] = None,
        limit: Annotated[
            Optional[StrictInt], Field(description="Number of items to return.")
        ] = None,
        index_prefix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name prefix. Only A/B tests for indices starting with this string are included in the response."
            ),
        ] = None,
        index_suffix: Annotated[
            Optional[StrictStr],
            Field(
                description="Index name suffix. Only A/B tests for indices ending with this string are included in the response."
            ),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ListABTestsResponse:
        """
        Lists all A/B tests you configured for this application.

        Required API Key ACLs:
          - analytics

        :param offset: Position of the first item to return.
        :type offset: int
        :param limit: Number of items to return.
        :type limit: int
        :param index_prefix: Index name prefix. Only A/B tests for indices starting with this string are included in the response.
        :type index_prefix: str
        :param index_suffix: Index name suffix. Only A/B tests for indices ending with this string are included in the response.
        :type index_suffix: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ListABTestsResponse' result object.
        """
        resp = self.list_ab_tests_with_http_info(
            offset, limit, index_prefix, index_suffix, request_options
        )
        return resp.deserialize(ListABTestsResponse, resp.raw_data)

    def schedule_ab_test_with_http_info(
        self,
        schedule_ab_tests_request: Union[ScheduleABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Schedule an A/B test to be started at a later time.

        Required API Key ACLs:
          - editSettings

        :param schedule_ab_tests_request: (required)
        :type schedule_ab_tests_request: ScheduleABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if schedule_ab_tests_request is None:
            raise ValueError(
                "Parameter `schedule_ab_tests_request` is required when calling `schedule_ab_test`."
            )

        _data = {}
        if schedule_ab_tests_request is not None:
            _data = schedule_ab_tests_request

        return self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/schedule",
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def schedule_ab_test(
        self,
        schedule_ab_tests_request: Union[ScheduleABTestsRequest, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ScheduleABTestResponse:
        """
        Schedule an A/B test to be started at a later time.

        Required API Key ACLs:
          - editSettings

        :param schedule_ab_tests_request: (required)
        :type schedule_ab_tests_request: ScheduleABTestsRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ScheduleABTestResponse' result object.
        """
        resp = self.schedule_ab_test_with_http_info(
            schedule_ab_tests_request, request_options
        )
        return resp.deserialize(ScheduleABTestResponse, resp.raw_data)

    def stop_ab_test_with_http_info(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Stops an A/B test by its ID.  You can't restart stopped A/B tests.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if id is None:
            raise ValueError("Parameter `id` is required when calling `stop_ab_test`.")

        return self._transporter.request(
            verb=Verb.POST,
            path="/2/abtests/{id}/stop".replace("{id}", quote(str(id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    def stop_ab_test(
        self,
        id: Annotated[StrictInt, Field(description="Unique A/B test identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ABTestResponse:
        """
        Stops an A/B test by its ID.  You can't restart stopped A/B tests.

        Required API Key ACLs:
          - editSettings

        :param id: Unique A/B test identifier. (required)
        :type id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'ABTestResponse' result object.
        """
        resp = self.stop_ab_test_with_http_info(id, request_options)
        return resp.deserialize(ABTestResponse, resp.raw_data)



from typing import Any, Dict, List

from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.hosts import Host
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.retry import RetryStrategy


class BaseTransporter:
    def __init__(self, config: BaseConfig) -> None:
        self._config = config
        self._retry_strategy = RetryStrategy()
        self._hosts: List[Host] = []
        self._timeout = 5000

    @property
    def config(self) -> BaseConfig:
        return self._config

    def prepare(
        self,
        request_options: RequestOptions,
        use_read_transporter: bool,
    ) -> Dict[str, Any]:
        query_parameters = dict(request_options.query_parameters)

        if use_read_transporter:
            self._timeout = request_options.timeouts["read"]
            self._hosts = (
                self._config.hosts.read() if self._config.hosts is not None else []
            )
            if isinstance(request_options.data, dict):
                query_parameters.update(request_options.data)
        else:
            self._timeout = request_options.timeouts["write"]
            self._hosts = (
                self._config.hosts.write() if self._config.hosts is not None else []
            )

        return query_parameters

    def build_path(self, path, query_parameters):
        if query_parameters is not None and len(query_parameters) > 0:
            return "{}?{}".format(
                path,
                "&".join(
                    [
                        "{}={}".format(key, value)
                        for key, value in query_parameters.items()
                    ]
                ),
            )
        return path

    def build_url(self, host, path):
        return "{}://{}{}".format(
            host.scheme,
            host.url + (":{}".format(host.port) if host.port else ""),
            path,
        )

    def get_proxy(self, url):
        if self._config.proxies is None:
            return None

        if url.startswith("https"):
            return self._config.proxies.get("https")
        elif url.startswith("http"):
            return self._config.proxies.get("http")
        else:
            return None

    def get_proxies(self, url):
        if self._config.proxies is None:
            return None

        if url.startswith("http"):
            return self._config.proxies
        else:
            return None



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.synonym_type import SynonymType

_ALIASES = {
    "object_id": "objectID",
    "type": "type",
    "synonyms": "synonyms",
    "input": "input",
    "word": "word",
    "corrections": "corrections",
    "placeholder": "placeholder",
    "replacements": "replacements",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class SynonymHit(BaseModel):
    """
    Synonym object.
    """

    object_id: str
    """ Unique identifier of a synonym object. """
    type: SynonymType
    synonyms: Optional[List[str]] = None
    """ Words or phrases considered equivalent. """
    input: Optional[str] = None
    """ Word or phrase to appear in query strings (for [`onewaysynonym`s](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/one-way-synonyms/)). """
    word: Optional[str] = None
    """ Word or phrase to appear in query strings (for [`altcorrection1` and `altcorrection2`](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-alternative-corrections/)). """
    corrections: Optional[List[str]] = None
    """ Words to be matched in records. """
    placeholder: Optional[str] = None
    """ [Placeholder token](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-placeholders/) to be put inside records.  """
    replacements: Optional[List[str]] = None
    """ Query words that will match the [placeholder token](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/adding-synonyms/in-depth/synonyms-placeholders/). """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SynonymHit from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SynonymHit from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["type"] = obj.get("type")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .delete_user_profile_response import DeleteUserProfileResponse
from .error_base import ErrorBase
from .event_type import EventType
from .events_scoring import EventsScoring
from .facets_scoring import FacetsScoring
from .get_user_token_response import GetUserTokenResponse
from .personalization_strategy_params import PersonalizationStrategyParams
from .set_personalization_strategy_response import SetPersonalizationStrategyResponse

__all__ = (
    "DeleteUserProfileResponse",
    "ErrorBase",
    "EventType",
    "EventsScoring",
    "FacetsScoring",
    "GetUserTokenResponse",
    "PersonalizationStrategyParams",
    "SetPersonalizationStrategyResponse",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional, Union

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.analytics.models.currency_code import CurrencyCode

_ALIASES = {
    "hit": "hit",
    "count": "count",
    "click_through_rate": "clickThroughRate",
    "conversion_rate": "conversionRate",
    "tracked_hit_count": "trackedHitCount",
    "click_count": "clickCount",
    "conversion_count": "conversionCount",
    "add_to_cart_rate": "addToCartRate",
    "add_to_cart_count": "addToCartCount",
    "purchase_rate": "purchaseRate",
    "purchase_count": "purchaseCount",
    "currencies": "currencies",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class TopHitWithRevenueAnalytics(BaseModel):
    """
    TopHitWithRevenueAnalytics
    """

    hit: str
    """ Object ID of a record returned as a search result. """
    count: int
    """ Number of occurrences. """
    click_through_rate: Union[float, None]
    """ Click-through rate: calculated as the number of tracked searches with at least one click event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true.  """
    conversion_rate: Union[float, None]
    """ Conversion rate: calculated as the number of tracked searches with at least one conversion event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true.  """
    tracked_hit_count: int
    """ Number of tracked searches. Tracked searches are search requests where the `clickAnalytics` parameter is true. """
    click_count: int
    """ Number of clicks associated with this search. """
    conversion_count: int
    """ Number of conversions from this search. """
    add_to_cart_rate: Union[float, None]
    """ Add-to-cart rate: calculated as the number of tracked searches with at least one add-to-cart event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true.  """
    add_to_cart_count: int
    """ Number of add-to-cart events from this search. """
    purchase_rate: Union[float, None]
    """ Purchase rate: calculated as the number of tracked searches with at least one purchase event divided by the number of tracked searches. If null, Algolia didn't receive any search requests with `clickAnalytics` set to true.  """
    purchase_count: int
    """ Number of purchase events from this search. """
    currencies: Dict[str, CurrencyCode]
    """ Revenue associated with this search: broken down by currency.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TopHitWithRevenueAnalytics from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TopHitWithRevenueAnalytics from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["currencies"] = (
            dict(
                (_k, CurrencyCode.from_dict(_v)) for _k, _v in obj["currencies"].items()
            )
            if obj.get("currencies") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, field_validator

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.insights.models.add_to_cart_event import AddToCartEvent
from algoliasearch.insights.models.conversion_event import ConversionEvent
from algoliasearch.insights.models.object_data_after_search import ObjectDataAfterSearch
from algoliasearch.insights.models.value import Value

_ALIASES = {
    "event_name": "eventName",
    "event_type": "eventType",
    "event_subtype": "eventSubtype",
    "index": "index",
    "query_id": "queryID",
    "object_ids": "objectIDs",
    "user_token": "userToken",
    "authenticated_user_token": "authenticatedUserToken",
    "currency": "currency",
    "object_data": "objectData",
    "timestamp": "timestamp",
    "value": "value",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class AddedToCartObjectIDsAfterSearch(BaseModel):
    """
    Use this event to track when users add items to their shopping cart after a previous Algolia request. If you're building your category pages with Algolia, you'll also use this event.
    """

    event_name: str
    """ Event name, up to 64 ASCII characters.  Consider naming events consistently—for example, by adopting Segment's [object-action](https://segment.com/academy/collecting-data/naming-conventions-for-clean-data/#the-object-action-framework) framework.  """
    event_type: ConversionEvent
    event_subtype: AddToCartEvent
    index: str
    """ Index name (case-sensitive) to which the event's items belong. """
    query_id: str
    """ Unique identifier for a search query.  The query ID is required for events related to search or browse requests. If you add `clickAnalytics: true` as a search request parameter, the query ID is included in the API response.  """
    object_ids: List[str]
    """ Object IDs of the records that are part of the event. """
    user_token: str
    """ Anonymous or pseudonymous user identifier.  Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/).  """
    authenticated_user_token: Optional[str] = None
    """ Identifier for authenticated users.  When the user signs in, you can get an identifier from your system and send it as `authenticatedUserToken`. This lets you keep using the `userToken` from before the user signed in, while providing a reliable way to identify users across sessions. Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/).  """
    currency: Optional[str] = None
    """ Three-letter [currency code](https://www.iso.org/iso-4217-currency-codes.html). """
    object_data: Optional[List[ObjectDataAfterSearch]] = None
    """ Extra information about the records involved in a purchase or add-to-cart events.  If provided, it must be the same length as `objectIDs`.  """
    timestamp: Optional[int] = None
    """ Timestamp of the event, measured in milliseconds since the Unix epoch. By default, the Insights API uses the time it receives an event as its timestamp.  """
    value: Optional[Value] = None

    @field_validator("event_name")
    def event_name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[\x20-\x7E]{1,64}", value):
            raise ValueError(
                r"must validate the regular expression /[\x20-\x7E]{1,64}/"
            )
        return value

    @field_validator("query_id")
    def query_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[0-9a-f]{32}", value):
            raise ValueError(r"must validate the regular expression /[0-9a-f]{32}/")
        return value

    @field_validator("user_token")
    def user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    @field_validator("authenticated_user_token")
    def authenticated_user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AddedToCartObjectIDsAfterSearch from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AddedToCartObjectIDsAfterSearch from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["eventType"] = obj.get("eventType")
        obj["eventSubtype"] = obj.get("eventSubtype")
        obj["objectData"] = (
            [ObjectDataAfterSearch.from_dict(_item) for _item in obj["objectData"]]
            if obj.get("objectData") is not None
            else None
        )
        obj["value"] = (
            Value.from_dict(obj["value"]) if obj.get("value") is not None else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


_ALIASES = {
    "facets_count": "facetsCount",
    "facet_values": "facetValues",
    "nb_hits": "nbHits",
    "rules_match": "rulesMatch",
    "typo": "typo",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Exhaustive(BaseModel):
    """
    Whether certain properties of the search response are calculated exhaustive (exact) or approximated.
    """

    facets_count: Optional[bool] = None
    """ Whether the facet count is exhaustive (`true`) or approximate (`false`). See the [related discussion](https://support.algolia.com/hc/en-us/articles/4406975248145-Why-are-my-facet-and-hit-counts-not-accurate-). """
    facet_values: Optional[bool] = None
    """ The value is `false` if not all facet values are retrieved. """
    nb_hits: Optional[bool] = None
    """ Whether the `nbHits` is exhaustive (`true`) or approximate (`false`). When the query takes more than 50ms to be processed, the engine makes an approximation. This can happen when using complex filters on millions of records, when typo-tolerance was not exhaustive, or when enough hits have been retrieved (for example, after the engine finds 10,000 exact matches). `nbHits` is reported as non-exhaustive whenever an approximation is made, even if the approximation didn’t, in the end, impact the exhaustivity of the query. """
    rules_match: Optional[bool] = None
    """ Rules matching exhaustivity. The value is `false` if rules were enable for this query, and could not be fully processed due a timeout. This is generally caused by the number of alternatives (such as typos) which is too large. """
    typo: Optional[bool] = None
    """ Whether the typo search was exhaustive (`true`) or approximate (`false`). An approximation is done when the typo search query part takes more than 10% of the query budget (ie. 5ms by default) to be processed (this can happen when a lot of typo alternatives exist for the query). This field will not be included when typo-tolerance is entirely disabled. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Exhaustive from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Exhaustive from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.composition.models.around_precision import AroundPrecision
from algoliasearch.composition.models.around_radius import AroundRadius
from algoliasearch.composition.models.facet_filters import FacetFilters
from algoliasearch.composition.models.inside_bounding_box import InsideBoundingBox
from algoliasearch.composition.models.numeric_filters import NumericFilters
from algoliasearch.composition.models.optional_filters import OptionalFilters
from algoliasearch.composition.models.supported_language import SupportedLanguage

_ALIASES = {
    "query": "query",
    "filters": "filters",
    "page": "page",
    "get_ranking_info": "getRankingInfo",
    "relevancy_strictness": "relevancyStrictness",
    "facet_filters": "facetFilters",
    "optional_filters": "optionalFilters",
    "numeric_filters": "numericFilters",
    "hits_per_page": "hitsPerPage",
    "around_lat_lng": "aroundLatLng",
    "around_lat_lng_via_ip": "aroundLatLngViaIP",
    "around_radius": "aroundRadius",
    "around_precision": "aroundPrecision",
    "minimum_around_radius": "minimumAroundRadius",
    "inside_bounding_box": "insideBoundingBox",
    "inside_polygon": "insidePolygon",
    "query_languages": "queryLanguages",
    "natural_languages": "naturalLanguages",
    "enable_rules": "enableRules",
    "rule_contexts": "ruleContexts",
    "user_token": "userToken",
    "click_analytics": "clickAnalytics",
    "analytics": "analytics",
    "analytics_tags": "analyticsTags",
    "enable_ab_test": "enableABTest",
    "enable_re_ranking": "enableReRanking",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Params(BaseModel):
    """
    Params
    """

    query: Optional[str] = None
    """ Search query. """
    filters: Optional[str] = None
    """ Filter expression to only include items that match the filter criteria in the response.  You can use these filter expressions:  - **Numeric filters.** `<facet> <op> <number>`, where `<op>` is one of `<`, `<=`, `=`, `!=`, `>`, `>=`. - **Ranges.** `<facet>:<lower> TO <upper>` where `<lower>` and `<upper>` are the lower and upper limits of the range (inclusive). - **Facet filters.** `<facet>:<value>` where `<facet>` is a facet attribute (case-sensitive) and `<value>` a facet value. - **Tag filters.** `_tags:<value>` or just `<value>` (case-sensitive). - **Boolean filters.** `<facet>: true | false`.  You can combine filters with `AND`, `OR`, and `NOT` operators with the following restrictions:  - You can only combine filters of the same type with `OR`.   **Not supported:** `facet:value OR num > 3`. - You can't use `NOT` with combinations of filters.   **Not supported:** `NOT(facet:value OR facet:value)` - You can't combine conjunctions (`AND`) with `OR`.   **Not supported:** `facet:value OR (facet:value AND facet:value)`  Use quotes around your filters, if the facet attribute name or facet value has spaces, keywords (`OR`, `AND`, `NOT`), or quotes. If a facet attribute is an array, the filter matches if it matches at least one element of the array.  For more information, see [Filters](https://www.algolia.com/doc/guides/managing-results/refine-results/filtering/).  """
    page: Optional[int] = None
    """ Page of search results to retrieve. """
    get_ranking_info: Optional[bool] = None
    """ Whether the search response should include detailed ranking information. """
    relevancy_strictness: Optional[int] = None
    facet_filters: Optional[FacetFilters] = None
    optional_filters: Optional[OptionalFilters] = None
    numeric_filters: Optional[NumericFilters] = None
    hits_per_page: Optional[int] = None
    """ Number of hits per page. """
    around_lat_lng: Optional[str] = None
    """ Coordinates for the center of a circle, expressed as a comma-separated string of latitude and longitude.  Only records included within a circle around this central location are included in the results. The radius of the circle is determined by the `aroundRadius` and `minimumAroundRadius` settings. This parameter is ignored if you also specify `insidePolygon` or `insideBoundingBox`.  """
    around_lat_lng_via_ip: Optional[bool] = None
    """ Whether to obtain the coordinates from the request's IP address. """
    around_radius: Optional[AroundRadius] = None
    around_precision: Optional[AroundPrecision] = None
    minimum_around_radius: Optional[int] = None
    """ Minimum radius (in meters) for a search around a location when `aroundRadius` isn't set. """
    inside_bounding_box: Optional[InsideBoundingBox] = None
    inside_polygon: Optional[List[List[float]]] = None
    """ Coordinates of a polygon in which to search.  Polygons are defined by 3 to 10,000 points. Each point is represented by its latitude and longitude. Provide multiple polygons as nested arrays. For more information, see [filtering inside polygons](https://www.algolia.com/doc/guides/managing-results/refine-results/geolocation/#filtering-inside-rectangular-or-polygonal-areas). This parameter is ignored if you also specify `insideBoundingBox`.  """
    query_languages: Optional[List[SupportedLanguage]] = None
    """ Languages for language-specific query processing steps such as plurals, stop-word removal, and word-detection dictionaries.  This setting sets a default list of languages used by the `removeStopWords` and `ignorePlurals` settings. This setting also sets a dictionary for word detection in the logogram-based [CJK](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/normalization/#normalization-for-logogram-based-languages-cjk) languages. To support this, you must place the CJK language **first**.  **You should always specify a query language.** If you don't specify an indexing language, the search engine uses all [supported languages](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/supported-languages/), or the languages you specified with the `ignorePlurals` or `removeStopWords` parameters. This can lead to unexpected search results. For more information, see [Language-specific configuration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/).  """
    natural_languages: Optional[List[SupportedLanguage]] = None
    """ ISO language codes that adjust settings that are useful for processing natural language queries (as opposed to keyword searches):  - Sets `removeStopWords` and `ignorePlurals` to the list of provided languages. - Sets `removeWordsIfNoResults` to `allOptional`. - Adds a `natural_language` attribute to `ruleContexts` and `analyticsTags`.  """
    enable_rules: Optional[bool] = None
    """ Whether to enable rules. """
    rule_contexts: Optional[List[str]] = None
    """ Assigns a rule context to the search query.  [Rule contexts](https://www.algolia.com/doc/guides/managing-results/rules/rules-overview/how-to/customize-search-results-by-platform/#whats-a-context) are strings that you can use to trigger matching rules.  """
    user_token: Optional[str] = None
    """ Unique pseudonymous or anonymous user identifier.  This helps with analytics and click and conversion events. For more information, see [user token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/).  """
    click_analytics: Optional[bool] = None
    """ Whether to include a `queryID` attribute in the response.  The query ID is a unique identifier for a search query and is required for tracking [click and conversion events](https://www.algolia.com/guides/sending-events/getting-started/).  """
    analytics: Optional[bool] = None
    """ Whether this search will be included in Analytics. """
    analytics_tags: Optional[List[str]] = None
    """ Tags to apply to the query for [segmenting analytics data](https://www.algolia.com/doc/guides/search-analytics/guides/segments/). """
    enable_ab_test: Optional[bool] = None
    """ Whether to enable A/B testing for this search. """
    enable_re_ranking: Optional[bool] = None
    """ Whether this search will use [Dynamic Re-Ranking](https://www.algolia.com/doc/guides/algolia-ai/re-ranking/).  This setting only has an effect if you activated Dynamic Re-Ranking for this index in the Algolia dashboard.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Params from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Params from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["facetFilters"] = (
            FacetFilters.from_dict(obj["facetFilters"])
            if obj.get("facetFilters") is not None
            else None
        )
        obj["optionalFilters"] = (
            OptionalFilters.from_dict(obj["optionalFilters"])
            if obj.get("optionalFilters") is not None
            else None
        )
        obj["numericFilters"] = (
            NumericFilters.from_dict(obj["numericFilters"])
            if obj.get("numericFilters") is not None
            else None
        )
        obj["aroundRadius"] = (
            AroundRadius.from_dict(obj["aroundRadius"])
            if obj.get("aroundRadius") is not None
            else None
        )
        obj["aroundPrecision"] = (
            AroundPrecision.from_dict(obj["aroundPrecision"])
            if obj.get("aroundPrecision") is not None
            else None
        )
        obj["insideBoundingBox"] = (
            InsideBoundingBox.from_dict(obj["insideBoundingBox"])
            if obj.get("insideBoundingBox") is not None
            else None
        )
        obj["queryLanguages"] = obj.get("queryLanguages")
        obj["naturalLanguages"] = obj.get("naturalLanguages")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.composition.models.banner_image import BannerImage
from algoliasearch.composition.models.banner_link import BannerLink

_ALIASES = {
    "image": "image",
    "link": "link",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Banner(BaseModel):
    """
    Banner with image and link to redirect users.
    """

    image: Optional[BannerImage] = None
    link: Optional[BannerLink] = None

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Banner from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Banner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["image"] = (
            BannerImage.from_dict(obj["image"])
            if obj.get("image") is not None
            else None
        )
        obj["link"] = (
            BannerLink.from_dict(obj["link"]) if obj.get("link") is not None else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.ingestion.models.mapping_field_directive import MappingFieldDirective

_ALIASES = {
    "id": "id",
    "enabled": "enabled",
    "trigger": "trigger",
    "field_directives": "fieldDirectives",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class MappingKitAction(BaseModel):
    """
    Describes how a destination object should be resolved by means of applying a set of directives.
    """

    id: Optional[str] = None
    """ ID to uniquely identify this action. """
    enabled: bool
    """ Whether this action has any effect. """
    trigger: str
    """ Condition which must be satisfied to apply the action. If this evaluates to false, the action is not applied, and the process attempts to apply the next action, if any. """
    field_directives: List[MappingFieldDirective]

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MappingKitAction from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MappingKitAction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["fieldDirectives"] = (
            [MappingFieldDirective.from_dict(_item) for _item in obj["fieldDirectives"]]
            if obj.get("fieldDirectives") is not None
            else None
        )

        return cls.model_validate(obj)
